-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local M = {}

local helper = require('scenario/scenariohelper')
local running = false
local playerWon = false
local scenario = nil

local function reset()
    running = false
    playerWon = false
end

local function fail(reason)
    reset()
    scenarios.finish({failed = reason})
end

local function success()
    local finalTime = scenario.timer
    local minutes = math.floor(finalTime / 60);
    local seconds = finalTime - (minutes * 60);
    local timeStr = ''
    if minutes > 0 then
        timeStr = string.format("%02.0f:%05.2f", minutes, seconds)
    else
        timeStr = string.format("%0.2f", seconds) .. 's'
    end
    local result = {msg = 'Well done, your time: '..timeStr}
    scenarios.finish(result)
    reset()
end

local function onRaceInit()
    helper.queueLuaCommandByName('aiCar', 'extensions.loadModule("dragAi")')
    local aiTargetPos = helper.getVehicleByName('wp_ai_target'):getPosition()
    helper.queueLuaCommandByName('aiCar', 'dragAi.setTarget('..dumps({x= aiTargetPos.x, y = aiTargetPos.y, z = aiTargetPos.z})..')')
    helper.queueLuaCommandByName('aiCar', 'dragAi.getReady()')
    helper.queueLuaCommandByName('scenario_player0', 'drivetrain.setShifterMode(2)')
end

local function onRaceStart()
    reset()

    helper.queueLuaCommandByName('aiCar', 'dragAi.go()')
    helper.trackVehicle('scenario_player0', 'playerCar')
    helper.trackVehicle('aiCar', 'aiCar')

    running = true
end

local function onRaceTick(raceTick)
    if running and scenario.timer >= 18 then
        fail("You lost...")
    end
end

local function onRaceWaypoint(w, vehicleName, waypointName, time)
    local playerDistance = helper.getDistanceBetweenSceneObjects('scenario_player0', 'wp_target_distance')
    local aiDistance = helper.getDistanceBetweenSceneObjects('aiCar', 'wp_target_distance')
    playerWon = playerDistance <= aiDistance
end

local function onRaceResult(time)
    if playerWon then
        success()
    else
        fail('You lost...')
    end
    helper.queueLuaCommandByName('aiCar', 'dragAi.reset()')
end

local function onScenarioChange(sc)
  scenario = sc
end

M.onRaceStart = onRaceStart
M.onRaceInit = onRaceInit
M.onRaceTick = onRaceTick
M.onRaceResult = onRaceResult
M.onScenarioChange = onScenarioChange
M.onRaceWaypoint = onRaceWaypoint

return M
